// Importing necessary libraries and modules
import fs from "fs";
import { themes } from "../themes/index";

const TARGET_FILE = "./themes/README.md";

/**
 * Generates the markdown for a theme.
 *
 * @param {string} theme Theme name.
 * @returns {string} Theme markdown string.
 */
function generateThemeMarkdown(theme: string): string {
  // Creates a markdown string for a given theme
  return `\`${theme}\` ![${theme}][${theme}]`;
}

/**
 * Generates the link to the theme image for a username and theme.
 *
 * @param {string} username GitHub username.
 * @param {string} theme Theme name.
 * @returns {string} Theme link string.
 */
function generateThemeLink(username: string, theme: string): string {
  // Creates a link to the theme image for a specific username and theme
  return `[${theme}]: https://github-readme-profile-alpha.vercel.app/api?username=${username}&theme=${theme}`;
}

/**
 * Generates the content for the README file with available themes and preview links.
 *
 * @param {string} username GitHub username.
 * @returns {string} README content string.
 */
function generateReadme(username: string): string {
  const availableThemes = Object.keys(themes);
  const itemsPerRow = 3;

  // Generate table rows with theme markdown
  let themesPreviewTable = "";
  for (let i = 0; i < availableThemes.length; i += itemsPerRow) {
    const themesSlice = availableThemes.slice(i, i + itemsPerRow);
    const row = themesSlice.map(theme => generateThemeMarkdown(theme)).join(" | ");
    themesPreviewTable += `| ${row} |\n`;
  }

  // Generate individual theme links
  let themesPreviewLink = "";
  for (let i = 0; i < availableThemes.length; i += 1) {
    const themesSlice = availableThemes.slice(i, i + 1);
    const row = themesSlice.map(theme => generateThemeLink(username, theme)).join("\n");
    themesPreviewLink += `${row}\n`;
  }

  // Final README content with themes and links
  const readmeContent = `<!-- DO NOT EDIT THIS FILE DIRECTLY -->
## Available Themes

With inbuilt themes, you can customize the look of the card without doing any manual customization.

Use \`?theme=THEME_NAME\` parameter like so :-

\`\`\`md
![GitHub Profile](https://gh-readme-profile.vercel.app/api?username=${username}&theme=dark)
\`\`\`

## Themes Preview

|                   |                   |                   |
| :---------------: | :---------------: | :---------------: |
${themesPreviewTable}

Want to add a new theme? Consider reading the [contribution guidelines](/CONTRIBUTING.md#-themes-contribution) :D

${themesPreviewLink}`;

  return readmeContent;
}

// GitHub username to generate the README
const username = "FajarKim";

// Generate the README content
const generatedReadme = generateReadme(username);

// Write the README content to the specified file
fs.writeFileSync(TARGET_FILE, generatedReadme);
